# frozen_string_literal: true

=begin ========================================================================
= Copyright (c) AgiliCity d.o.o. 2025. All Rights Reserved.                   =
=                                                                             =
= By using this software, you agree to respect Terms and Conditions, End User =
= License Agreement and other related documents that can be found on our      =
= website (https://modelur.com/legal/).                                       =
=                                                                             =
= Make sure this file is not encoded by Ruby Encoder:                         =
= RubyEncoder:DO_NOT_ENCODE                                                   =
=end ==========================================================================
require 'rbconfig'
require 'pathname'
require_relative 'src/ui/sketchup_ui'
require_relative 'src/constants/error_constants'
require_relative 'src/constants/url_constants'

module Modelur
  dir = __dir__.dup
  dir.force_encoding('UTF-8') if dir.respond_to?(:force_encoding)
  MODELUR_SRC_PATH = Pathname.new(File.expand_path(dir)).cleanpath.to_s
  IMG_PATH = Pathname.new(File.join(MODELUR_SRC_PATH, 'img')).cleanpath.to_s
  ICON_EXT = case RbConfig::CONFIG['host_os']
             when /darwin|mac os/
               '.pdf'
             else
               '.svg'
             end

  module BootstrapFallback
    module_function

    def initialize_menu_and_toolbar
      @sketchup_ui = Ui::SketchupUi.new(nil)
      add_modelur_command
    end

    def add_modelur_command
      menu_text = 'Modelur Problem'
      cmd = UI::Command.new(menu_text) do
        display_error(LOAD_ERROR)
      end
      icon = File.join(IMG_PATH, "su_initialize_modelur#{ICON_EXT}")
      cmd.large_icon = icon
      cmd.small_icon = icon
      cmd.tooltip = menu_text
      cmd.status_bar_text = 'Modelur encountered a problem while loading. Please click this button to learn more.'
      @sketchup_ui.modelur_menu.add_item(cmd)
      @sketchup_ui.modelur_toolbar.add_item(cmd)
    end

    def display_error(load_error)
      case load_error.message
      when ERR_MODELUR_EXPIRED
        modelur_expired
      when ERR_INCOMPATIBLE_SU_VERSION
        incompatible_su_version
      when *ERR_SOLIDIFY_ERRORS
        solidify_problem(load_error)
      else
        unknown_reason(load_error)
      end
    end

    def modelur_expired
      date = Time.at(MODELUR_EXPIRATION).strftime('%b-%d-%Y')
      msg = 'Your installed version of Modelur (%<version>s) expired on %<date>s. '
      msg += "In order to keep on using it, you will need to upgrade Modelur to a newer version.\n\n"
      msg += 'Would you like to download the latest Modelur RBZ installer package?'
      status = UI.messagebox(format(msg, version: MODELUR_VERSION_EXTERNAL, date: date), MB_YESNO)
      return unless status == IDYES

      UI.openURL('https://modelur.com/download-latest-rbz-installer')
      msg =  'After you have downloaded Modelur RBZ installer, please install it'
      msg += " using SketchUp Extension Manager. We will open it for you when you click OK button below.\n\n"
      msg += 'Once you complete the installation, please restart SketchUp.'
      UI.messagebox(msg)
      UI.show_extension_manager
    end

    def incompatible_su_version
      msg =  "Modelur %<mdlr_version>s can not run on your version of SketchUp (20%<su_version>s).\n\n"
      msg += "Supported versions for this release of Modelur are from SketchUp 20%<su_min>s to SketchUp 20%<su_max>s.\n\n"
      msg += 'Please make sure you use supported version of SketchUp.'
      UI.messagebox(format(msg, su_min: MIN_SU_VERSION, su_max: MAX_SU_VERSION,
                                su_version: Sketchup.version, mdlr_version: MODELUR_VERSION_EXTERNAL))
    end

    def solidify_problem(error)
      msg =  "Modelur detected unusual SketchUp environment.\n\n"
      # msg += "REASON: #{error.message}.\n\n"
      msg += "REASON: Modelur Classes have been compromised. Error code #12.\n\n"
      msg += "To keep things running smoothly, Modelur won't load until the issue is resolved.\n\n"
      msg += "This problem is probably caused by some other Extension. Here's how to fix it:\n"
      msg += "   1. Click Yes button below to open Extension Manager.\n"
      msg += "   2. Temporarily disable (all) other Extensions.\n"
      msg += "   3. Restart SketchUp to see if Modelur is able to load.\n"
      msg += "   4a. If Modelur loads:\n"
      msg += "          Re-enable Extensions one-by-one to identify the culprit.\n"
      msg += "   4a. If Modelur doesn't load:\n"
      msg += "          Click No button below to show detailed information.\n\n"
      msg += "Help us improve! If you find a conflicting Extension or the issue persists, please let us know at support@modelur.com.\n\n"
      msg += "Please select your action below:\n"
      msg += "    YES to open Extension Manager.\n"
      msg += "    NO to show detailed error log (to send it to Modelur Support)\n"
      msg += "    CANCEL to stop resolving the issue.\n"
      status = UI.messagebox(msg, MB_YESNOCANCEL)
      return if status == IDCANCEL

      if status == IDYES
        UI.show_extension_manager
      else
        show_error(error)
      end
    end

    def unknown_reason(error)
      msg = "Modelur was not able to load due to unknown reason.\n\n"
      show_error(error, msg)
    end

    def show_error(error, msg = '')
      msg += "Please help us improve Modelur by sending us this error log report. To do so, do the following:\n"
      msg += "   1. Right-click inside this Window and choose \"Select All\".\n"
      msg += "   2. Right-click inside window again and choose \"Copy\.\n"
      msg += "   3. Paste the copied Error Log into a new email and send it to support@modelur.com.\n"
      msg += "\t\t\tThank you in advance!\n"
      msg += "\t\t\t   The Modelur Team\n\n"
      msg += "Error Log:\n"
      msg += error.full_message
      UI.messagebox(msg, MB_MULTILINE)
    end
  end

  BootstrapFallback.initialize_menu_and_toolbar
end
